@@ -6,6 +6,7 @@
  *  eventloop based on a heap insertion would be O(log N).
  */
 
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -16,6 +17,8 @@
 #include "duktape.h"
 #include "c_eventloop.h"
 
+#define  DUKTAPE_EVENTLOOP_DEBUG 0       /* set to 1 to debug with printf */
+
 #define  MAX_TIMERS             4096     /* this is quite excessive for embedded use, but good for testing */
 #define  MIN_DELAY              1.0
 #define  MIN_WAIT               1.0
@@ -55,6 +58,10 @@ static int poll_count = 0;
 /* Misc */
 static int exit_requested = 0;
 
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+static int duktape_debug(const char* fmt, ...);
+#endif
+
 /* Get Javascript compatible 'now' timestamp (millisecs since 1970). */
 static double get_now(void) {
     struct timeval tv;
@@ -126,9 +133,8 @@ static void expire_timers(duk_context *ctx) {
          */
 
         if (exit_requested) {
-#if 0
-            fprintf(stderr, "exit requested, exiting timer expiry loop\n");
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("exit requested, exiting timer expiry loop, last timer id %d\n", (int) timer_next_id);
 #endif
             break;
         }
@@ -169,27 +175,24 @@ static void expire_timers(duk_context *ctx) {
          *  need to worry about the timer's offset changing on the timer list.
          */
 
-#if 0
-        fprintf(stderr, "calling user callback for timer id %d\n", (int) t->id);
-        fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+        duktape_debug("calling user callback for timer id %d\n", (int) t->id);
 #endif
 
         duk_push_number(ctx, (double) t->id);
         duk_get_prop(ctx, -2);  /* -> [ ... stash eventTimers func ] */
         rc = duk_pcall(ctx, 0 /*nargs*/);  /* -> [ ... stash eventTimers retval ] */
         if (rc != 0) {
-#if 0
-            fprintf(stderr, "timer callback failed for timer %d: %s\n", (int) t->id, duk_to_string(ctx, -1));
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("timer callback failed for timer %d: %s\n", (int) t->id, duk_to_string(ctx, -1));
 #endif
         }
         duk_pop(ctx);    /* ignore errors for now -> [ ... stash eventTimers ] */
 
         if (t->removed) {
             /* One-shot timer (always removed) or removed by user callback. */
-#if 0
-            fprintf(stderr, "deleting callback state for timer %d\n", (int) t->id);
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("deleting callback state for timer %d\n", (int) t->id);
 #endif
             duk_push_number(ctx, (double) t->id);
             duk_del_prop(ctx, -2);
@@ -197,9 +200,8 @@ static void expire_timers(duk_context *ctx) {
             /* Interval timer, not removed by user callback.  Queue back to
              * timer list and bubble to its final sorted position.
              */
-#if 0
-            fprintf(stderr, "queueing timer %d back into active list\n", (int) t->id);
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("queueing timer %d back into active list\n", (int) t->id);
 #endif
             if (timer_count >= MAX_TIMERS) {
                 (void) duk_error(ctx, DUK_ERR_RANGE_ERROR, "out of timer slots");
@@ -227,18 +229,16 @@ static void compact_poll_list(void) {
         struct pollfd *pfd = poll_list + i;
         if (pfd->fd == 0) {
             /* keep output index the same */
-#if 0
-            fprintf(stderr, "remove pollfd (index %d): fd=%d, events=%d, revents=%d\n",
-                    i, pfd->fd, pfd->events, pfd->revents),
-                fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("remove pollfd (index %d): fd=%d, events=%d, revents=%d\n",
+                    i, pfd->fd, pfd->events, pfd->revents);
 #endif
 
             continue;
         }
-#if 0
-        fprintf(stderr, "keep pollfd (index %d -> %d): fd=%d, events=%d, revents=%d\n",
-                i, j, pfd->fd, pfd->events, pfd->revents),
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+        duktape_debug("keep pollfd (index %d -> %d): fd=%d, events=%d, revents=%d\n",
+                i, j, pfd->fd, pfd->events, pfd->revents);
 #endif
         if (i != j) {
             /* copy only if indices have diverged */
@@ -288,9 +288,8 @@ duk_ret_t eventloop_run(duk_context *ctx, void *udata) {
          */
 
         if (exit_requested) {
-#if 0
-            fprintf(stderr, "exit requested, exiting event loop\n");
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("exit requested, exiting event loop, last timer id %d\n", (int) timer_next_id);
 #endif
             break;
         }
@@ -318,9 +317,8 @@ duk_ret_t eventloop_run(duk_context *ctx, void *udata) {
             timeout = (int) diff;  /* clamping ensures that fits */
         } else {
             if (poll_count == 0) {
-#if 0
-                fprintf(stderr, "no timers and no sockets to poll, exiting\n");
-                fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+                duktape_debug("no timers and no sockets to poll, exiting event loop, last timer id %d\n", (int) timer_next_id);
 #endif
                 break;
             }
@@ -331,15 +329,13 @@ duk_ret_t eventloop_run(duk_context *ctx, void *udata) {
          *  Poll for activity or timeout.
          */
 
-#if 0
-        fprintf(stderr, "going to poll, timeout %d ms, pollfd count %d\n", timeout, poll_count);
-        fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+        duktape_debug("going to poll, timeout %d ms, pollfd count %d\n", timeout, poll_count);
 #endif
 
         rc = poll(poll_list, poll_count, timeout);
-#if 0
-        fprintf(stderr, "poll rc: %d\n", rc);
-        fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+        duktape_debug("poll rc: %d\n", rc);
 #endif
         if (rc < 0) {
             /* error */
@@ -369,9 +365,8 @@ duk_ret_t eventloop_run(duk_context *ctx, void *udata) {
             }
 
             if (pfd->revents) {
-#if 0
-                fprintf(stderr, "fd %d has revents: %d\n", (int) pfd->fd, (int) pfd->revents);
-                fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+                duktape_debug("fd %d has revents: %d\n", (int) pfd->fd, (int) pfd->revents);
 #endif
                 duk_dup(ctx, idx_fd_handler);
                 duk_dup(ctx, idx_eventloop);
@@ -379,9 +374,8 @@ duk_ret_t eventloop_run(duk_context *ctx, void *udata) {
                 duk_push_int(ctx, pfd->revents);
                 rc = duk_pcall_method(ctx, 2 /*nargs*/);
                 if (rc) {
-#if 0
-                    fprintf(stderr, "fd callback failed for fd %d: %s\n", (int) pfd->fd, duk_to_string(ctx, -1));
-                    fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+                    duktape_debug("fd callback failed for fd %d: %s\n", (int) pfd->fd, duk_to_string(ctx, -1));
 #endif
                 }
                 duk_pop(ctx);
@@ -450,9 +444,8 @@ static int create_timer(duk_context *ctx) {
     /* Return timer id. */
 
     duk_push_number(ctx, (double) timer_id);
-#if 0
-    fprintf(stderr, "created timer id: %d\n", (int) timer_id);
-    fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+    duktape_debug("created timer id: %d\n", (int) timer_id);
 #endif
     return 1;
 }
@@ -484,9 +477,8 @@ static int delete_timer(duk_context *ctx) {
     if (t->id == timer_id) {
         t->removed = 1;
         duk_push_true(ctx);
-#if 0
-        fprintf(stderr, "deleted expiring timer id: %d\n", (int) timer_id);
-        fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+        duktape_debug("deleted expiring timer id: %d\n", (int) timer_id);
 #endif
         return 1;
     }
@@ -519,18 +511,16 @@ static int delete_timer(duk_context *ctx) {
             duk_push_number(ctx, (double) timer_id);
             duk_del_prop(ctx, -2);  /* delete eventTimers[timer_id] */
 
-#if 0
-            fprintf(stderr, "deleted timer id: %d\n", (int) timer_id);
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("deleted timer id: %d\n", (int) timer_id);
 #endif
             break;
         }
     }
 
-#if 0
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
     if (!found) {
-        fprintf(stderr, "trying to delete timer id %d, but not found; ignoring\n", (int) timer_id);
-        fflush(stderr);
+        duktape_debug("trying to delete timer id %d, but not found; ignoring\n", (int) timer_id);
     }
 #endif
 
@@ -544,9 +534,8 @@ static int listen_fd(duk_context *ctx) {
     int i, n;
     struct pollfd *pfd;
 
-#if 0
-    fprintf(stderr, "listen_fd: fd=%d, events=%d\n", fd, events);
-    fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+    duktape_debug("listen_fd: fd=%d, events=%d\n", fd, events);
 #endif
     /* events == 0 means stop listening to the FD */
 
@@ -554,9 +543,8 @@ static int listen_fd(duk_context *ctx) {
     for (i = 0; i < n; i++) {
         pfd = poll_list + i;
         if (pfd->fd == fd) {
-#if 0
-            fprintf(stderr, "listen_fd: fd found at index %d\n", i);
-            fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+            duktape_debug("listen_fd: fd found at index %d\n", i);
 #endif
             if (events == 0) {
                 /* mark to-be-deleted, cleaned up by next poll */
@@ -569,9 +557,8 @@ static int listen_fd(duk_context *ctx) {
     }
 
     /* not found, append to list */
-#if 0
-    fprintf(stderr, "listen_fd: fd not found on list, add new entry\n");
-    fflush(stderr);
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+    duktape_debug("listen_fd: fd not found on list, add new entry\n");
 #endif
 
     if (poll_count >= MAX_FDS) {
@@ -619,3 +606,17 @@ void eventloop_register(duk_context *ctx) {
     duk_put_prop_string(ctx, -2, "eventTimers");
     duk_pop(ctx);
 }
+
+#if defined(DUKTAPE_EVENTLOOP_DEBUG) && DUKTAPE_EVENTLOOP_DEBUG > 0
+static int duktape_debug(const char* fmt, ...)
+{
+    va_list ap;
+    va_start(ap, fmt);
+    double now = get_now();
+    fprintf(stderr, "%.3f ", now);
+    int n = vfprintf(stderr, fmt, ap);
+    fflush(stderr);
+    va_end(ap);
+    return n;
+}
+#endif
